@@ -227,8 +227,12 @@ bool CmdLineParser::parseFromArgs(int argc, const char* const argv[])
             else if (std::strcmp(argv[i], "--check-library") == 0)
                 mSettings->checkLibrary = true;
 
-            else if (std::strcmp(argv[i], "--clang") == 0)
+            else if (std::strncmp(argv[i], "--clang", 7) == 0) {
                 mSettings->clang = true;
+                if (std::strncmp(argv[i], "--clang=", 8) == 0) {
+                    mSettings->clangExecutable = argv[i] + 8;
+                }
+            }
 
             else if (std::strncmp(argv[i], "--config-exclude=",17) ==0) {
                 mSettings->configExcludePaths.insert(Path::fromNativeSeparators(argv[i] + 17));
@@ -824,7 +828,7 @@ bool CmdLineParser::parseFromArgs(int argc, const char* const argv[])
                     mSettings->templateLocation = "{file}:{line}:{column}: note: {info}\\n{code}";
                 } else if (mSettings->templateFormat == "daca2") {
                     mSettings->daca = true;
-                    mSettings->templateFormat = "{file}:{line}:{column}: {severity}: {message} [{id}]";
+                    mSettings->templateFormat = "{file}:{line}:{column}: {severity}:{inconclusive:inconclusive:} {message} [{id}]";
                     mSettings->templateLocation = "{file}:{line}:{column}: note: {info}";
                 } else if (mSettings->templateFormat == "vs")
                     mSettings->templateFormat = "{file}({line}): {severity}: {message}";
@@ -952,23 +956,33 @@ void CmdLineParser::printHelp()
               "Options:\n"
               "    --addon=<addon>\n"
               "                         Execute addon. i.e. --addon=cert. If options must be\n"
-              "                         provided a json configuration is needed, see the manual\n"
-              "                         for additional information about that.\n"
+              "                         provided a json configuration is needed.\n"
               "    --addon-python=<python interpreter>\n"
-              "                         Use this python interpreter to run addons.\n"
+              "                         You can specify the python interpreter either in the\n"
+              "                         addon json files or through this command line option.\n"
               "                         If not present, Cppcheck will try \"python3\" first and\n"
               "                         then \"python\".\n"
               "    --bug-hunting\n"
-              "                         Noisy and soundy analysis.\n"
+              "                         Enable noisy and soundy analysis. The normal Cppcheck\n"
+              "                         analysis is turned off.\n"
               "    --cppcheck-build-dir=<dir>\n"
-              "                         Analysis output directory. Useful for various data.\n"
-              "                         Some possible usages are; whole program analysis,\n"
-              "                         incremental analysis, distributed analysis.\n"
+              "                         Cppcheck work folder. Advantages:\n"
+              "                          * whole program analysis\n"
+              "                          * faster analysis; Cppcheck will reuse the results if\n"
+              "                            the hash for a file is unchanged.\n"
+              "                          * some useful debug information, i.e. commands used to\n"
+              "                            execute clang/clang-tidy/addons.\n"
               "    --check-config       Check cppcheck configuration. The normal code\n"
               "                         analysis is disabled by this flag.\n"
               "    --check-library      Show information messages when library files have\n"
               "                         incomplete info.\n"
-              "    --clang              Import Clang AST\n"
+              "    --clang=<path>       Experimental: Use Clang parser instead of the builtin Cppcheck\n"
+              "                         parser. Takes the executable as optional parameter and\n"
+              "                         defaults to `clang`. Cppcheck will run the given Clang\n"
+              "                         executable, import the Clang AST and convert it into\n"
+              "                         Cppcheck data. After that the normal Cppcheck analysis is\n"
+              "                         used. You must have the executable in PATH if no path is\n"
+              "                         given.\n"
               "    --config-exclude=<dir>\n"
               "                         Path (prefix) to be excluded from configuration\n"
               "                         checking. Preprocessor configurations defined in\n"
@@ -976,7 +990,6 @@ void CmdLineParser::printHelp()
               "                         be considered for evaluation.\n"
               "    --config-excludes-file=<file>\n"
               "                         A file that contains a list of config-excludes\n"
-              "    --doc                Print a list of all available checks.\n"
               "    --dump               Dump xml data for each translation unit. The dump\n"
               "                         files have the extension .dump and contain ast,\n"
               "                         tokenlist, symboldatabase, valueflow.\n"
@@ -984,9 +997,6 @@ void CmdLineParser::printHelp()
               "                         --force is used, Cppcheck will only check the given\n"
               "                         configuration when -D is used.\n"
               "                         Example: '-DDEBUG=1 -D__cplusplus'.\n"
-              "    -U<ID>               Undefine preprocessor symbol. Use -U to explicitly\n"
-              "                         hide certain #ifdef <ID> code paths from checking.\n"
-              "                         Example: '-UDEBUG'\n"
               "    -E                   Print preprocessor output on stdout and don't do any\n"
               "                         further processing.\n"
               "    --enable=<id>        Enable additional checks. The available ids are:\n"
@@ -1117,7 +1127,6 @@ void CmdLineParser::printHelp()
               "    --plist-output=<path>\n"
               "                         Generate Clang-plist output files in folder.\n"
               "    -q, --quiet          Do not show progress reports.\n"
-              "    -rp, --relative-paths\n"
               "    -rp=<paths>, --relative-paths=<paths>\n"
               "                         Use relative paths in output. When given, <paths> are\n"
               "                         used as base. You can separate multiple paths by ';'.\n"
@@ -1198,11 +1207,12 @@ void CmdLineParser::printHelp()
               "                           \\r         insert carriage return\n"
               "                         Example format (gcc-like):\n"
               "                         '{file}:{line}:{column}: note: {info}\\n{code}'\n"
+              "    -U<ID>               Undefine preprocessor symbol. Use -U to explicitly\n"
+              "                         hide certain #ifdef <ID> code paths from checking.\n"
+              "                         Example: '-UDEBUG'\n"
               "    -v, --verbose        Output more detailed error information.\n"
               "    --version            Print out version number.\n"
               "    --xml                Write results in xml format to error stream (stderr).\n"
-              "    --xml-version=<version>\n"
-              "                         Select the XML file version. Currently only versions 2 is available."
               "\n"
               "Example usage:\n"
               "  # Recursively check the current folder. Print the progress on the screen and\n"
