@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2019 Cppcheck team.
+ * Copyright (C) 2007-2020 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,10 +21,10 @@
 
 #include <QDialog>
 #include <QString>
-#include "application.h"
 #include "ui_application.h"
 
 class QWidget;
+class Application;
 
 /// @addtogroup GUI
 /// @{
