@@ -50,11 +50,8 @@ public:
     virtual ~ResultsView();
     ResultsView &operator=(const ResultsView &) = delete;
 
-    void setTags(const QStringList &tags) {
-        mUI.mTree->setTags(tags);
-    }
-
-    void setAddedContracts(const QStringList &addedContracts);
+    void setAddedFunctionContracts(const QStringList &addedContracts);
+    void setAddedVariableContracts(const QStringList &added);
 
     /**
      * @brief Clear results and statistics and reset progressinfo.
@@ -72,6 +69,9 @@ public:
      */
     void clearRecheckFile(const QString &filename);
 
+    /** Clear the contracts */
+    void clearContracts();
+
     /**
     * @brief Write statistics in file
     *
@@ -200,6 +200,9 @@ public:
         return &mUI.mTree->mShowSeverities;
     }
 
+    /** Show/hide the contract tabs */
+    void showContracts(bool visible);
+
 signals:
 
     /**
@@ -222,17 +225,21 @@ signals:
     */
     void checkSelected(QStringList selectedFilesList);
 
-    /**
-     * Some results have been tagged
-     */
-    void tagged();
-
     /** Suppress Ids */
     void suppressIds(QStringList ids);
 
     /** Edit contract for function */
     void editFunctionContract(QString function);
 
+    /** Delete contract for function */
+    void deleteFunctionContract(QString function);
+
+    /** Edit contract for variable */
+    void editVariableContract(QString var);
+
+    /** Delete variable contract */
+    void deleteVariableContract(QString var);
+
     /**
     * @brief Show/hide certain type of errors
     * Refreshes the tree.
@@ -351,6 +358,11 @@ public slots:
     /** \brief Contract was double clicked => edit it */
     void contractDoubleClicked(QListWidgetItem* item);
 
+    /** \brief Variable was double clicked => edit it */
+    void variableDoubleClicked(QListWidgetItem* item);
+
+    void editVariablesFilter(const QString &text);
+
 protected:
     /**
     * @brief Should we show a "No errors found dialog" every time no errors were found?
@@ -360,6 +372,8 @@ protected:
     Ui::ResultsView mUI;
 
     CheckStatistics *mStatistics;
+
+    bool eventFilter(QObject *target, QEvent *event);
 private slots:
     /**
      * @brief Custom context menu for Analysis Log
@@ -367,7 +381,11 @@ private slots:
      */
     void on_mListLog_customContextMenuRequested(const QPoint &pos);
 private:
-    QSet<QString> mContracts;
+    QSet<QString> mFunctionContracts;
+    QSet<QString> mVariableContracts;
+
+    /** Current file shown in the code editor */
+    QString mCurrentFileName;
 };
 /// @}
 #endif // RESULTSVIEW_H
