@@ -54,7 +54,16 @@ bool CheckCondition::diag(const Token* tok, bool insert)
 {
     if (!tok)
         return false;
-    if (mCondDiags.find(tok) == mCondDiags.end()) {
+    const Token* parent = tok->astParent();
+    bool hasParent = false;
+    while (Token::Match(parent, "&&|%oror%")) {
+        if (mCondDiags.count(parent) != 0) {
+            hasParent = true;
+            break;
+        }
+        parent = parent->astParent();
+    }
+    if (mCondDiags.count(tok) == 0 && !hasParent) {
         if (insert)
             mCondDiags.insert(tok);
         return false;
@@ -452,7 +461,13 @@ void CheckCondition::duplicateCondition()
             continue;
 
         bool modified = false;
-        visitAstNodes(cond1, [&](const Token *tok3) {
+        visitAstNodes(cond1, [&](const Token* tok3) {
+            if (exprDependsOnThis(tok3)) {
+                if (isThisChanged(scope.classDef->next(), cond2, false, mSettings, mTokenizer->isCPP())) {
+                    modified = true;
+                    return ChildrenToVisit::done;
+                }
+            }
             if (tok3->varId() > 0 &&
                 isVariableChanged(scope.classDef->next(), cond2, tok3->varId(), false, mSettings, mTokenizer->isCPP())) {
                 modified = true;
@@ -682,17 +697,14 @@ void CheckCondition::multiCondition2()
                     ErrorPath errorPath;
 
                     if (type == MULTICONDITIONTYPE::INNER) {
-                        std::stack<const Token *> tokens1;
-                        tokens1.push(cond1);
-                        while (!tokens1.empty()) {
-                            const Token *firstCondition = tokens1.top();
-                            tokens1.pop();
+                        visitAstNodes(cond1, [&](const Token* firstCondition) {
                             if (!firstCondition)
-                                continue;
+                                return ChildrenToVisit::none;
                             if (firstCondition->str() == "&&") {
-                                tokens1.push(firstCondition->astOperand1());
-                                tokens1.push(firstCondition->astOperand2());
-                            } else if (!firstCondition->hasKnownIntValue()) {
+                                if (!isOppositeCond(false, mTokenizer->isCPP(), firstCondition, cond2, mSettings->library, true, true))
+                                    return ChildrenToVisit::op1_and_op2;
+                            }
+                            if (!firstCondition->hasKnownIntValue()) {
                                 if (!isReturnVar && isOppositeCond(false, mTokenizer->isCPP(), firstCondition, cond2, mSettings->library, true, true, &errorPath)) {
                                     if (!isAliased(vars))
                                         oppositeInnerConditionError(firstCondition, cond2, errorPath);
@@ -700,24 +712,22 @@ void CheckCondition::multiCondition2()
                                     identicalInnerConditionError(firstCondition, cond2, errorPath);
                                 }
                             }
-                        }
+                            return ChildrenToVisit::none;
+                        });
                     } else {
-                        std::stack<const Token *> tokens2;
-                        tokens2.push(cond2);
-                        while (!tokens2.empty()) {
-                            const Token *secondCondition = tokens2.top();
-                            tokens2.pop();
-                            if (!secondCondition)
-                                continue;
-                            if (secondCondition->str() == "||" || secondCondition->str() == "&&") {
-                                tokens2.push(secondCondition->astOperand1());
-                                tokens2.push(secondCondition->astOperand2());
-                            } else if ((!cond1->hasKnownIntValue() || !secondCondition->hasKnownIntValue()) &&
-                                       isSameExpression(mTokenizer->isCPP(), true, cond1, secondCondition, mSettings->library, true, true, &errorPath)) {
-                                if (!isAliased(vars))
+                        visitAstNodes(cond2, [&](const Token *secondCondition) {
+                            if (secondCondition->str() == "||" || secondCondition->str() == "&&")
+                                return ChildrenToVisit::op1_and_op2;
+
+                            if ((!cond1->hasKnownIntValue() || !secondCondition->hasKnownIntValue()) &&
+                                isSameExpression(mTokenizer->isCPP(), true, cond1, secondCondition, mSettings->library, true, true, &errorPath)) {
+                                if (!isAliased(vars) && !mTokenizer->hasIfdef(cond1, secondCondition)) {
                                     identicalConditionAfterEarlyExitError(cond1, secondCondition, errorPath);
+                                    return ChildrenToVisit::done;
+                                }
                             }
-                        }
+                            return ChildrenToVisit::none;
+                        });
                     }
                 }
                 if (Token::Match(tok, "%name% (") && isVariablesChanged(tok, tok->linkAt(1), true, varsInCond, mSettings, mTokenizer->isCPP())) {
@@ -1120,7 +1130,7 @@ void CheckCondition::checkIncorrectLogicOperator()
 
             // Comparison #1 (LHS)
             const Token *comp1 = tok->astOperand1();
-            if (comp1 && comp1->str() == tok->str())
+            if (comp1->str() == tok->str())
                 comp1 = comp1->astOperand2();
 
             // Comparison #2 (RHS)
@@ -1240,6 +1250,8 @@ void CheckCondition::checkIncorrectLogicOperator()
 
 void CheckCondition::incorrectLogicOperatorError(const Token *tok, const std::string &condition, bool always, bool inconclusive, ErrorPath errors)
 {
+    if (diag(tok))
+        return;
     errors.emplace_back(tok, "");
     if (always)
         reportError(errors, Severity::warning, "incorrectLogicOperator",
@@ -1255,6 +1267,8 @@ void CheckCondition::incorrectLogicOperatorError(const Token *tok, const std::st
 
 void CheckCondition::redundantConditionError(const Token *tok, const std::string &text, bool inconclusive)
 {
+    if (diag(tok))
+        return;
     reportError(tok, Severity::style, "redundantCondition", "Redundant condition: " + text, CWE398, inconclusive);
 }
 
@@ -1401,6 +1415,8 @@ void CheckCondition::alwaysTrueFalse()
                     condition = parent->astOperand1();
                 else if (Token::Match(parent->previous(), "if|while ("))
                     condition = parent->astOperand2();
+                else if (Token::simpleMatch(parent, "return"))
+                    condition = parent->astOperand1();
                 else if (parent->str() == ";" && parent->astParent() && parent->astParent()->astParent() && Token::simpleMatch(parent->astParent()->astParent()->previous(), "for ("))
                     condition = parent->astOperand1();
                 else
@@ -1427,26 +1443,22 @@ void CheckCondition::alwaysTrueFalse()
             const bool constValExpr = tok->isNumber() && Token::Match(tok->astParent(),"%oror%|&&|?"); // just one number in boolean expression
             const bool compExpr = Token::Match(tok, "%comp%|!"); // a compare expression
             const bool ternaryExpression = Token::simpleMatch(tok->astParent(), "?");
+            const bool returnExpression = Token::simpleMatch(tok->astTop(), "return") && (tok->isComparisonOp() || Token::Match(tok, "&&|%oror%"));
 
-            if (!(constIfWhileExpression || constValExpr || compExpr || ternaryExpression))
+            if (!(constIfWhileExpression || constValExpr || compExpr || ternaryExpression || returnExpression))
                 continue;
 
             // Don't warn when there are expanded macros..
             bool isExpandedMacro = false;
-            std::stack<const Token*> tokens;
-            tokens.push(tok);
-            while (!tokens.empty()) {
-                const Token *tok2 = tokens.top();
-                tokens.pop();
+            visitAstNodes(tok, [&](const Token * tok2) {
                 if (!tok2)
-                    continue;
-                tokens.push(tok2->astOperand1());
-                tokens.push(tok2->astOperand2());
+                    return ChildrenToVisit::none;
                 if (tok2->isExpandedMacro()) {
                     isExpandedMacro = true;
-                    break;
+                    return ChildrenToVisit::done;
                 }
-            }
+                return ChildrenToVisit::op1_and_op2;
+            });
             if (isExpandedMacro)
                 continue;
             for (const Token *parent = tok; parent; parent = parent->astParent()) {
@@ -1460,25 +1472,23 @@ void CheckCondition::alwaysTrueFalse()
 
             // don't warn when condition checks sizeof result
             bool hasSizeof = false;
-            tokens.push(tok);
-            while (!tokens.empty()) {
-                const Token *tok2 = tokens.top();
-                tokens.pop();
+            bool hasNonNumber = false;
+            visitAstNodes(tok, [&](const Token * tok2) {
                 if (!tok2)
-                    continue;
+                    return ChildrenToVisit::none;
                 if (tok2->isNumber())
-                    continue;
+                    return ChildrenToVisit::none;
                 if (Token::simpleMatch(tok2->previous(), "sizeof (")) {
                     hasSizeof = true;
-                    continue;
+                    return ChildrenToVisit::none;
                 }
                 if (tok2->isComparisonOp() || tok2->isArithmeticalOp()) {
-                    tokens.push(tok2->astOperand1());
-                    tokens.push(tok2->astOperand2());
+                    return ChildrenToVisit::op1_and_op2;
                 } else
-                    break;
-            }
-            if (tokens.empty() && hasSizeof)
+                    hasNonNumber = true;
+                return ChildrenToVisit::none;
+            });
+            if (!hasNonNumber && hasSizeof)
                 continue;
 
             alwaysTrueFalseError(tok, &tok->values().front());
