@@ -21,6 +21,7 @@
 #include "checkunusedvar.h"
 
 #include "astutils.h"
+#include "preprocessor.h"
 #include "settings.h"
 #include "symboldatabase.h"
 #include "token.h"
@@ -44,6 +45,53 @@ namespace {
 static const struct CWE CWE563(563U);   // Assignment to Variable without Use ('Unused Variable')
 static const struct CWE CWE665(665U);   // Improper Initialization
 
+/** Is scope a raii class scope */
+static bool isRaiiClassScope(const Scope *classScope)
+{
+    return classScope && classScope->getDestructor() != nullptr;
+}
+
+/** Is ValueType a raii class? */
+static bool isRaiiClass(const ValueType *valueType, bool cpp, bool defaultReturn = true)
+{
+    if (!cpp)
+        return false;
+
+    if (!valueType)
+        return defaultReturn;
+
+    if (valueType->smartPointerType && isRaiiClassScope(valueType->smartPointerType->classScope))
+        return true;
+
+    switch (valueType->type) {
+    case ValueType::Type::UNKNOWN_TYPE:
+    case ValueType::Type::NONSTD:
+        return defaultReturn;
+
+    case ValueType::Type::RECORD:
+        if (isRaiiClassScope(valueType->typeScope))
+            return true;
+        return defaultReturn;
+
+    case ValueType::Type::CONTAINER:
+    case ValueType::Type::ITERATOR:
+    case ValueType::Type::VOID:
+    case ValueType::Type::BOOL:
+    case ValueType::Type::CHAR:
+    case ValueType::Type::SHORT:
+    case ValueType::Type::WCHAR_T:
+    case ValueType::Type::INT:
+    case ValueType::Type::LONG:
+    case ValueType::Type::LONGLONG:
+    case ValueType::Type::UNKNOWN_INT:
+    case ValueType::Type::FLOAT:
+    case ValueType::Type::DOUBLE:
+    case ValueType::Type::LONGDOUBLE:
+        return false;
+    }
+
+    return defaultReturn;
+}
 
 /**
  * @brief This class is used create a list of variables within a function.
@@ -244,8 +292,8 @@ void Variables::readAliases(unsigned int varid, const Token* tok)
     VariableUsage *usage = find(varid);
 
     if (usage) {
-        for (std::set<unsigned int>::iterator aliases = usage->_aliases.begin(); aliases != usage->_aliases.end(); ++aliases) {
-            VariableUsage *aliased = find(*aliases);
+        for (unsigned int aliases : usage->_aliases) {
+            VariableUsage *aliased = find(aliases);
 
             if (aliased) {
                 aliased->_read = true;
@@ -1131,38 +1179,16 @@ void CheckUnusedVar::checkFunctionVariableUsage()
             const bool isIncrementOrDecrement = (tok->tokType() == Token::Type::eIncDecOp);
             if (!isAssignment && !isInitialization && !isIncrementOrDecrement)
                 continue;
+
+            if (isIncrementOrDecrement && tok->astParent() && precedes(tok, tok->astOperand1()))
+                continue;
+
+            if (tok->str() == "=" && isRaiiClass(tok->valueType(), mTokenizer->isCPP(), false))
+                continue;
+
             if (tok->isName()) {
-                if (mTokenizer->isCPP()) {
-                    // do not check RAII/scope_lock objects
-                    if (!tok->valueType())
-                        continue;
-                    bool check = false;
-                    switch (tok->valueType()->type) {
-                    case ValueType::Type::UNKNOWN_TYPE:
-                    case ValueType::Type::NONSTD:
-                    case ValueType::Type::RECORD:
-                        check = tok->valueType()->typeScope && !tok->valueType()->typeScope->getDestructor();
-                        break;
-                    case ValueType::Type::CONTAINER:
-                    case ValueType::Type::ITERATOR:
-                    case ValueType::Type::VOID:
-                    case ValueType::Type::BOOL:
-                    case ValueType::Type::CHAR:
-                    case ValueType::Type::SHORT:
-                    case ValueType::Type::WCHAR_T:
-                    case ValueType::Type::INT:
-                    case ValueType::Type::LONG:
-                    case ValueType::Type::LONGLONG:
-                    case ValueType::Type::UNKNOWN_INT:
-                    case ValueType::Type::FLOAT:
-                    case ValueType::Type::DOUBLE:
-                    case ValueType::Type::LONGDOUBLE:
-                        check = true;
-                        break;
-                    }
-                    if (!check)
-                        continue;
-                }
+                if (isRaiiClass(tok->valueType(), mTokenizer->isCPP(), true))
+                    continue;
                 tok = tok->next();
             }
             if (tok->astParent() && tok->str() != "(") {
@@ -1205,6 +1231,10 @@ void CheckUnusedVar::checkFunctionVariableUsage()
                 if (op1Var->nameToken()->isAttributeUnused())
                     continue;
 
+                // Avoid FP for union..
+                if (op1Var->type() && op1Var->type()->isUnionType())
+                    continue;
+
                 // Bailout for unknown template classes, we have no idea what side effects such assignments have
                 if (mTokenizer->isCPP() &&
                     op1Var->isClass() &&
@@ -1228,6 +1258,9 @@ void CheckUnusedVar::checkFunctionVariableUsage()
 
             const Token *expr = varDecl ? varDecl : tok->astOperand1();
 
+            if (isInitialization)
+                expr = tok->previous();
+
             // Is variable in lhs a union member?
             if (tok->previous() && tok->previous()->variable() && tok->previous()->variable()->nameToken()->scope()->type == Scope::eUnion)
                 continue;
@@ -1340,6 +1373,17 @@ void CheckUnusedVar::checkStructMemberUsage()
         // Packed struct => possibly used by lowlevel code. Struct members might be required by hardware.
         if (scope.bodyEnd->isAttributePacked())
             continue;
+        if (const Preprocessor *preprocessor = mTokenizer->getPreprocessor()) {
+            bool isPacked = false;
+            for (const Directive &d: preprocessor->getDirectives()) {
+                if (d.str == "#pragma pack(1)" && d.file == mTokenizer->list.getFiles().front() && d.linenr < scope.bodyStart->linenr()) {
+                    isPacked=true;
+                    break;
+                }
+            }
+            if (isPacked)
+                continue;
+        }
 
         // Bail out if struct/union contains any functions
         if (!scope.functionList.empty())
@@ -1430,24 +1474,50 @@ bool CheckUnusedVar::isRecordTypeWithoutSideEffects(const Type* type)
 
     const std::pair<std::map<const Type *,bool>::iterator,bool> found=mIsRecordTypeWithoutSideEffectsMap.insert(
                 std::pair<const Type *,bool>(type,false)); //Initialize with side effects for possible recursions
-    bool & withoutSideEffects=found.first->second;
+    bool & withoutSideEffects = found.first->second;
     if (!found.second)
         return withoutSideEffects;
 
-    if (type && type->classScope && type->classScope->numConstructors == 0 &&
-        (type->classScope->varlist.empty() || type->needInitialization == Type::NeedInitialization::True)) {
-        for (std::vector<Type::BaseInfo>::const_iterator i = type->derivedFrom.begin(); i != type->derivedFrom.end(); ++i) {
-            if (!isRecordTypeWithoutSideEffects(i->type)) {
-                withoutSideEffects=false;
-                return withoutSideEffects;
-            }
-        }
-        withoutSideEffects=true;
-        return withoutSideEffects;
+    // unknown types are assumed to have side effects
+    if (!type || !type->classScope)
+        return (withoutSideEffects = false);
+
+    // Non-empty constructors => possible side effects
+    for (const Function& f : type->classScope->functionList) {
+        if (!f.isConstructor())
+            continue;
+        if (f.argDef && Token::simpleMatch(f.argDef->link(), ") ="))
+            continue; // ignore default/deleted constructors
+        const bool emptyBody = (f.functionScope && Token::simpleMatch(f.functionScope->bodyStart, "{ }"));
+        const bool hasInitList = f.argDef && Token::simpleMatch(f.argDef->link(), ") :");
+        if (!emptyBody || hasInitList)
+            return (withoutSideEffects = false);
     }
 
-    withoutSideEffects=false;   // unknown types are assumed to have side effects
-    return withoutSideEffects;
+    // Derived from type that has side effects?
+    for (const Type::BaseInfo& derivedFrom : type->derivedFrom) {
+        if (!isRecordTypeWithoutSideEffects(derivedFrom.type))
+            return (withoutSideEffects = false);
+    }
+
+    // Is there a member variable with possible side effects
+    for (const Variable& var : type->classScope->varlist) {
+        if (var.isPointer())
+            continue;
+
+        const Type* variableType = var.type();
+        if (variableType) {
+            if (!isRecordTypeWithoutSideEffects(variableType))
+                return (withoutSideEffects = false);
+        } else {
+            ValueType::Type valueType = var.valueType()->type;
+            if ((valueType == ValueType::Type::UNKNOWN_TYPE) || (valueType == ValueType::Type::NONSTD))
+                return (withoutSideEffects = false);
+        }
+    }
+
+
+    return (withoutSideEffects = true);
 }
 
 bool CheckUnusedVar::isEmptyType(const Type* type)
