@@ -168,21 +168,22 @@ static std::vector<std::string> split(const std::string &str, const std::string
 {
     std::vector<std::string> ret;
     for (std::string::size_type startPos = 0U; startPos < str.size();) {
-        std::string::size_type endPos;
-        if (str[startPos] == '\"') {
-            endPos = str.find("\"", startPos + 1);
-            if (endPos < str.size())
-                endPos++;
-        } else {
-            endPos = str.find(sep, startPos + 1);
-        }
-        if (endPos == std::string::npos) {
-            ret.push_back(str.substr(startPos));
+        startPos = str.find_first_not_of(sep, startPos);
+        if (startPos == std::string::npos)
             break;
+
+        if (str[startPos] == '\"') {
+            const std::string::size_type endPos = str.find("\"", startPos + 1);
+            ret.push_back(str.substr(startPos + 1, endPos - startPos - 1));
+            startPos = (endPos < str.size()) ? (endPos + 1) : endPos;
+            continue;
         }
+
+        const std::string::size_type endPos = str.find(sep, startPos + 1);
         ret.push_back(str.substr(startPos, endPos - startPos));
-        startPos = str.find_first_not_of(sep, endPos);
+        startPos = endPos;
     }
+
     return ret;
 }
 
@@ -201,14 +202,14 @@ static std::string executeAddon(const AddonInfo &addonInfo,
         pythonExe = cmdFileName(defaultPythonExe);
     else {
 #ifdef _WIN32
-        const char *p[] = { "python3.exe", "python.exe" };
+        const char *py_exes[] = { "python3.exe", "python.exe" };
 #else
-        const char *p[] = { "python3", "python" };
+        const char *py_exes[] = { "python3", "python" };
 #endif
-        for (int i = 0; i < 2; ++i) {
+        for (const char* py_exe : py_exes) {
             std::string out;
-            if (executeCommand(p[i], split("--version"), redirect, &out) && out.compare(0, 7, "Python ") == 0 && std::isdigit(out[7])) {
-                pythonExe = p[i];
+            if (executeCommand(py_exe, split("--version"), redirect, &out) && out.compare(0, 7, "Python ") == 0 && std::isdigit(out[7])) {
+                pythonExe = py_exe;
                 break;
             }
         }
@@ -327,21 +328,17 @@ unsigned int CppCheck::check(const std::string &path)
         const std::string analyzerInfo = mSettings.buildDir.empty() ? std::string() : AnalyzerInformation::getAnalyzerInfoFile(mSettings.buildDir, path, "");
         const std::string clangcmd = analyzerInfo + ".clang-cmd";
         const std::string clangStderr = analyzerInfo + ".clang-stderr";
+        std::string exe = mSettings.clangExecutable;
 #ifdef _WIN32
-        const std::string exe = "clang.exe";
-#else
-        const std::string exe = "clang";
+        // append .exe if it is not a path
+        if (Path::fromNativeSeparators(mSettings.clangExecutable).find('/') == std::string::npos) {
+            exe += ".exe";
+        }
 #endif
 
         std::string flags(lang + " ");
-        if (Path::isCPP(path)) {
-            if (mSettings.standards.cpp == Standards::CPP14)
-                flags += "-std=c++14 ";
-            else if (mSettings.standards.cpp == Standards::CPP17)
-                flags += "-std=c++17 ";
-            else if (mSettings.standards.cpp == Standards::CPP20)
-                flags += "-std=c++20 ";
-        }
+        if (Path::isCPP(path) && !mSettings.standards.stdValue.empty())
+            flags += "-std=" + mSettings.standards.stdValue + " ";
 
         for (const std::string &i: mSettings.includePaths)
             flags += "-I" + i + " ";
@@ -412,12 +409,10 @@ unsigned int CppCheck::check(const ImportProject::FileSettings &fs)
         temp.mSettings.userDefines += fs.cppcheckDefines();
     temp.mSettings.includePaths = fs.includePaths;
     temp.mSettings.userUndefs = fs.undefs;
-    if (fs.standard == "c++14")
-        temp.mSettings.standards.cpp = Standards::CPP14;
-    else if (fs.standard == "c++17")
-        temp.mSettings.standards.cpp = Standards::CPP17;
-    else if (fs.standard == "c++20")
-        temp.mSettings.standards.cpp = Standards::CPP20;
+    if (fs.standard.find("++") != std::string::npos)
+        temp.mSettings.standards.setCPP(fs.standard);
+    else if (!fs.standard.empty())
+        temp.mSettings.standards.setC(fs.standard);
     if (fs.platformType != Settings::Unspecified)
         temp.mSettings.platform(fs.platformType);
     if (mSettings.clang) {
@@ -688,16 +683,17 @@ unsigned int CppCheck::checkFile(const std::string& filename, const std::string
                 continue;
             }
 
-            Tokenizer mTokenizer(&mSettings, this);
+            Tokenizer tokenizer(&mSettings, this);
+            tokenizer.setPreprocessor(&preprocessor);
             if (mSettings.showtime != SHOWTIME_MODES::SHOWTIME_NONE)
-                mTokenizer.setTimerResults(&s_timerResults);
+                tokenizer.setTimerResults(&s_timerResults);
 
             try {
                 // Create tokens, skip rest of iteration if failed
                 {
                     Timer timer("Tokenizer::createTokens", mSettings.showtime, &s_timerResults);
                     simplecpp::TokenList tokensP = preprocessor.preprocess(tokens1, mCurrentConfig, files, true);
-                    mTokenizer.createTokens(std::move(tokensP));
+                    tokenizer.createTokens(std::move(tokensP));
                 }
                 hasValidConfig = true;
 
@@ -708,7 +704,7 @@ unsigned int CppCheck::checkFile(const std::string& filename, const std::string
                     mErrorLogger.reportOut("Checking " + fixedpath + ": " + mCurrentConfig + "...");
                 }
 
-                if (!mTokenizer.tokens())
+                if (!tokenizer.tokens())
                     continue;
 
                 // skip rest of iteration if just checking configuration
@@ -716,11 +712,11 @@ unsigned int CppCheck::checkFile(const std::string& filename, const std::string
                     continue;
 
                 // Check raw tokens
-                checkRawTokens(mTokenizer);
+                checkRawTokens(tokenizer);
 
                 // Simplify tokens into normal form, skip rest of iteration if failed
                 Timer timer2("Tokenizer::simplifyTokens1", mSettings.showtime, &s_timerResults);
-                bool result = mTokenizer.simplifyTokens1(mCurrentConfig);
+                bool result = tokenizer.simplifyTokens1(mCurrentConfig);
                 timer2.stop();
                 if (!result)
                     continue;
@@ -733,13 +729,13 @@ unsigned int CppCheck::checkFile(const std::string& filename, const std::string
                     fdump << "    <cpp version=\"" << mSettings.standards.getCPP() << "\"/>" << std::endl;
                     fdump << "  </standards>" << std::endl;
                     preprocessor.dump(fdump);
-                    mTokenizer.dump(fdump);
+                    tokenizer.dump(fdump);
                     fdump << "</dump>" << std::endl;
                 }
 
                 // Skip if we already met the same simplified token list
                 if (mSettings.force || mSettings.maxConfigs > 1) {
-                    const unsigned long long checksum = mTokenizer.list.calculateChecksum();
+                    const unsigned long long checksum = tokenizer.list.calculateChecksum();
                     if (checksums.find(checksum) != checksums.end()) {
                         if (mSettings.debugwarnings)
                             purgedConfigurationMessage(filename, mCurrentConfig);
@@ -749,11 +745,11 @@ unsigned int CppCheck::checkFile(const std::string& filename, const std::string
                 }
 
                 // Check normal tokens
-                checkNormalTokens(mTokenizer);
+                checkNormalTokens(tokenizer);
 
                 // Analyze info..
                 if (!mSettings.buildDir.empty())
-                    checkUnusedFunctions.parseTokens(mTokenizer, filename.c_str(), &mSettings);
+                    checkUnusedFunctions.parseTokens(tokenizer, filename.c_str(), &mSettings);
 
                 // simplify more if required, skip rest of iteration if failed
                 if (mSimplify && hasRule("simple")) {
@@ -761,13 +757,13 @@ unsigned int CppCheck::checkFile(const std::string& filename, const std::string
 
                     // if further simplification fails then skip rest of iteration
                     Timer timer3("Tokenizer::simplifyTokenList2", mSettings.showtime, &s_timerResults);
-                    result = mTokenizer.simplifyTokenList2();
+                    result = tokenizer.simplifyTokenList2();
                     timer3.stop();
                     if (!result)
                         continue;
 
                     if (!Settings::terminated())
-                        executeRules("simple", mTokenizer);
+                        executeRules("simple", tokenizer);
                 }
 
             } catch (const simplecpp::Output &o) {
@@ -779,16 +775,16 @@ unsigned int CppCheck::checkFile(const std::string& filename, const std::string
             } catch (const InternalError &e) {
                 std::list<ErrorMessage::FileLocation> locationList;
                 if (e.token) {
-                    ErrorMessage::FileLocation loc(e.token, &mTokenizer.list);
+                    ErrorMessage::FileLocation loc(e.token, &tokenizer.list);
                     locationList.push_back(loc);
                 } else {
-                    ErrorMessage::FileLocation loc(mTokenizer.list.getSourceFilePath(), 0, 0);
+                    ErrorMessage::FileLocation loc(tokenizer.list.getSourceFilePath(), 0, 0);
                     ErrorMessage::FileLocation loc2(filename, 0, 0);
                     locationList.push_back(loc2);
                     locationList.push_back(loc);
                 }
                 ErrorMessage errmsg(locationList,
-                                    mTokenizer.list.getSourceFilePath(),
+                                    tokenizer.list.getSourceFilePath(),
                                     Severity::error,
                                     e.errorMessage,
                                     e.id,
