@@ -22,6 +22,7 @@
 //---------------------------------------------------------------------------
 
 #include "config.h"
+#include "errortypes.h"
 
 #include <functional>
 #include <map>
@@ -78,9 +79,17 @@ namespace ExprEngine {
         virtual std::string getNewSymbolName() = 0;
         const std::string currentFunction;
         const Settings * const settings;
-        virtual void addError(int linenr) {
-            (void)linenr;
-        }
+        virtual bool isC() const = 0;
+        virtual bool isCPP() const = 0;
+        virtual void reportError(const Token *tok,
+                                 Severity::SeverityType severity,
+                                 const char id[],
+                                 const std::string &text,
+                                 CWE cwe,
+                                 bool inconclusive,
+                                 bool incomplete=false,
+                                 const std::string &functionName = std::string()) = 0;
+        ErrorPath errorPath;
     };
 
     class Value {
@@ -134,7 +143,8 @@ namespace ExprEngine {
         IntRange(const std::string &name, int128_t minValue, int128_t maxValue)
             : Value(name, ValueType::IntRange)
             , minValue(minValue)
-            , maxValue(maxValue) {
+            , maxValue(maxValue)
+            , loopScope(nullptr) {
         }
         std::string getRange() const OVERRIDE {
             if (minValue == maxValue)
@@ -147,6 +157,7 @@ namespace ExprEngine {
 
         int128_t minValue;
         int128_t maxValue;
+        const Scope *loopScope;
     };
 
     class FloatRange : public Value {
@@ -187,6 +198,7 @@ namespace ExprEngine {
 
         ArrayValue(const std::string &name, ValuePtr size, ValuePtr value, bool pointer, bool nullPointer, bool uninitPointer);
         ArrayValue(DataBase *data, const Variable *var);
+        ArrayValue(const std::string &name, const ArrayValue &arrayValue);
 
         std::string getRange() const OVERRIDE;
         std::string getSymbolicExpression() const OVERRIDE;
@@ -204,7 +216,7 @@ namespace ExprEngine {
             ValuePtr value;
         };
         std::vector<IndexAndValue> data;
-        ValuePtr size;
+        std::vector<ValuePtr> size;
     };
 
     class StringLiteralValue: public Value {
@@ -231,6 +243,15 @@ namespace ExprEngine {
             auto it = member.find(n);
             return (it == member.end()) ? ValuePtr() : it->second;
         }
+
+        std::string getUninitStructMember() const {
+            for (auto memberNameValue: member) {
+                if (memberNameValue.second && memberNameValue.second->isUninit())
+                    return memberNameValue.first;
+            }
+            return std::string();
+        }
+
         std::map<std::string, ValuePtr> member;
     };
 
@@ -296,11 +317,9 @@ namespace ExprEngine {
         bool isEqual(DataBase * /*dataBase*/, int /*value*/) const OVERRIDE {
             return true;
         }
-        /* FIXME: This is too noisy
         bool isUninit() const OVERRIDE {
             return true;
         }
-        */
     };
 
     typedef std::function<void(const Token *, const ExprEngine::Value &, ExprEngine::DataBase *)> Callback;
