@@ -162,7 +162,7 @@ void ImportProject::FileSettings::setIncludePaths(const std::string &basepath, c
             continue;
         if (it.compare(0,2,"%(")==0)
             continue;
-        std::string s(Path::fromNativeSeparators(it));
+        std::string s(Path::removeQuotationMarks(Path::fromNativeSeparators(it)));
         if (s[0] == '/' || (s.size() > 1U && s.compare(1,2,":/") == 0)) {
             if (!endsWith(s,'/'))
                 s += '/';
@@ -217,13 +217,13 @@ ImportProject::Type ImportProject::import(const std::string &filename, Settings
     return ImportProject::Type::UNKNOWN;
 }
 
-static std::string readUntil(const std::string &command, std::string::size_type *pos, const char until[])
+static std::string readUntil(const std::string& command, std::string::size_type* pos, const char until[], bool skipBackSlash = true)
 {
     std::string ret;
     bool str = false;
     while (*pos < command.size() && (str || !std::strchr(until, command[*pos]))) {
-        if (command[*pos] == '\\')
-            ++*pos;
+        if (skipBackSlash && command[*pos] == '\\')
+            ++* pos;
         if (*pos < command.size())
             ret += command[(*pos)++];
         if (endsWith(ret, '\"'))
@@ -232,112 +232,126 @@ static std::string readUntil(const std::string &command, std::string::size_type
     return ret;
 }
 
+static void skip_whitespaces(const std::string& command, std::string::size_type *pos)
+{
+    while (*pos < command.size() && command[*pos] == ' ')
+        (*pos)++;
+}
+
+
+static std::string parseTillNextCommandOpt(const std::string& singleCharOpts, const std::string& command, std::string::size_type *pos)
+{
+    std::string ret;
+    *pos = command.find_first_not_of("/-", *pos);
+
+    if (*pos >= command.size())
+        return ret;
+
+    const char F = command[*pos];
+    if (std::strchr(singleCharOpts.c_str(), F)) {
+        (*pos)++;
+        return std::string{F};
+    }
+
+    ret = readUntil(command, pos, " =");
+    return ret;
+}
+
+void ImportProject::FileSettings::parseCommandStd(const std::string& command, std::string::size_type *pos, std::string& defs)
+{
+    std::string def{};
+    const std::string stdval = readUntil(command, pos, " ");
+    standard = stdval;
+    if (standard.compare(0, 3, "c++") || standard.compare(0, 5, "gnu++")) {
+        std::string stddef;
+        if (standard == "c++98" || standard == "gnu++98" || standard == "c++03" || standard == "gnu++03") {
+            stddef = "199711L";
+        } else if (standard == "c++11" || standard == "gnu++11"|| standard == "c++0x" || standard == "gnu++0x") {
+            stddef = "201103L";
+        } else if (standard == "c++14" || standard == "gnu++14" || standard == "c++1y" || standard == "gnu++1y") {
+            stddef = "201402L";
+        } else if (standard == "c++17" || standard == "gnu++17" || standard == "c++1z" || standard == "gnu++1z") {
+            stddef = "201703L";
+        }
+
+        if (stddef.empty()) {
+            // TODO: log error
+        } else {
+            def += "__cplusplus=";
+            def += stddef;
+            def += ";";
+        }
+    }
+    defs += def;
+}
+
+void ImportProject::FileSettings::parseCommandDefine(const std::string& command, std::string::size_type *pos, std::string& defs)
+{
+    const bool skipBackSlash = false;
+    defs += readUntil(command, pos, " =");
+    const std::string defval = readUntil(command, pos, " ", skipBackSlash);
+    if (!defval.empty())
+        defs += defval;
+    defs += ';';
+}
+
+void ImportProject::FileSettings::parseCommandUndefine(const std::string& command,  std::string::size_type *pos)
+{
+    const std::string fval = readUntil(command, pos, " ");
+    undefs.insert(fval);
+}
+
+void ImportProject::FileSettings::parseCommandInclude(const std::string& command,  std::string::size_type *pos)
+{
+    const bool skipBackSlash = false;
+    const std::string fval = readUntil(command, pos, " ", skipBackSlash);
+    if (std::find(includePaths.begin(), includePaths.end(), fval) == includePaths.end())
+        includePaths.push_back(fval);
+}
+
+void ImportProject::FileSettings::parseCommandSystemInclude(const std::string& command,  std::string::size_type *pos)
+{
+    const bool skipBackSlash = false;
+    const std::string isystem = Path::removeQuotationMarks(readUntil(command, pos, " ", skipBackSlash));
+    systemIncludePaths.push_back(isystem);
+}
+
 void ImportProject::FileSettings::parseCommand(const std::string &command)
 {
+    const std::string singleCharCommandOpts = "DUI";
     std::string defs;
 
-    // Parse command..
     std::string::size_type pos = 0;
     while (std::string::npos != (pos = command.find(' ',pos))) {
-        while (pos < command.size() && command[pos] == ' ')
-            pos++;
+        skip_whitespaces(command, &pos);
         if (pos >= command.size())
             break;
-        if (command[pos] != '/' && command[pos] != '-')
-            continue;
-        pos++;
+
+        const auto opt = parseTillNextCommandOpt(singleCharCommandOpts, command, &pos);
+
         if (pos >= command.size())
             break;
-        const char F = command[pos++];
-        if (std::strchr("DUI", F)) {
-            while (pos < command.size() && command[pos] == ' ')
-                ++pos;
-        }
-        const std::string fval = readUntil(command, &pos, " =");
-        if (F=='D') {
-            const std::string defval = readUntil(command, &pos, " ");
-            defs += fval;
-            if (!defval.empty())
-                defs += defval;
-            defs += ';';
-        } else if (F=='U')
-            undefs.insert(fval);
-        else if (F=='I') {
-            if (std::find(includePaths.begin(), includePaths.end(), fval) == includePaths.end())
-                includePaths.push_back(fval);
-        } else if (F=='s' && fval.compare(0,2,"td") == 0) {
-            ++pos;
-            const std::string stdval = readUntil(command, &pos, " ");
-            standard = stdval;
-            if (standard.compare(0, 3, "c++") || standard.compare(0, 5, "gnu++")) {
-                std::string stddef;
-                if (standard == "c++98" || standard == "gnu++98" || standard == "c++03" || standard == "gnu++03") {
-                    stddef = "199711L";
-                } else if (standard == "c++11" || standard == "gnu++11"|| standard == "c++0x" || standard == "gnu++0x") {
-                    stddef = "201103L";
-                } else if (standard == "c++14" || standard == "gnu++14" || standard == "c++1y" || standard == "gnu++1y") {
-                    stddef = "201402L";
-                } else if (standard == "c++17" || standard == "gnu++17" || standard == "c++1z" || standard == "gnu++1z") {
-                    stddef = "201703L";
-                }
 
-                if (stddef.empty()) {
-                    // TODO: log error
-                    continue;
-                }
-
-                defs += "__cplusplus=";
-                defs += stddef;
-                defs += ";";
-            } else if (standard.compare(0, 1, "c") || standard.compare(0, 3, "gnu")) {
-                if (standard == "c90" || standard == "iso9899:1990" || standard == "gnu90" || standard == "iso9899:199409") {
-                    // __STDC_VERSION__ is not set for C90 although the macro was added in the 1994 amendments
-                    continue;
-                }
-
-                std::string stddef;
-
-                if (standard == "c99" || standard == "iso9899:1999" || standard == "gnu99") {
-                    stddef = "199901L";
-                } else if (standard == "c11" || standard == "iso9899:2011" || standard == "gnu11" || standard == "c1x" || standard == "gnu1x") {
-                    stddef = "201112L";
-                } else if (standard == "c17") {
-                    stddef = "201710L";
-                }
-
-                if (stddef.empty()) {
-                    // TODO: log error
-                    continue;
-                }
-
-                defs += "__STDC_VERSION__=";
-                defs += stddef;
-                defs += ";";
-            }
-        } else if (F == 'i' && fval == "system") {
-            ++pos;
-            const std::string isystem = readUntil(command, &pos, " ");
-            systemIncludePaths.push_back(isystem);
-        } else if (F=='m') {
-            if (fval == "unicode") {
-                defs += "UNICODE";
-                defs += ";";
-            }
-        } else if (F=='f') {
-            if (fval == "pic") {
-                defs += "__pic__";
-                defs += ";";
-            } else if (fval == "PIC") {
-                defs += "__PIC__";
-                defs += ";";
-            } else if (fval == "pie") {
-                defs += "__pie__";
-                defs += ";";
-            } else if (fval == "PIE") {
-                defs += "__PIE__";
-                defs += ";";
-            }
-        }
+        if (opt=="D")
+            parseCommandDefine(command, &pos, defs);
+        else if (opt=="U")
+            parseCommandUndefine(command, &pos);
+        else if (opt=="I")
+            parseCommandInclude(command, &pos);
+        else if (opt=="isystem")
+            parseCommandSystemInclude(command, &pos);
+        else if (opt=="std")
+            parseCommandStd(command, &pos, defs);
+        else if (opt=="municode")
+            defs += "UNICODE;";
+        else if (opt=="fpic")
+            defs += "__pic__;";
+        else if (opt=="fPIC")
+            defs += "__PIC__;";
+        else if (opt=="fpie")
+            defs += "__pie__;";
+        else if (opt=="fPIE")
+            defs += "__PIE__;";
     }
     setDefines(defs);
 }
@@ -1033,7 +1047,7 @@ bool ImportProject::importCppcheckGuiProject(std::istream &istr, Settings *setti
     const std::string &path = mPath;
 
     std::list<std::string> paths;
-    std::list<std::string> suppressions;
+    std::list<Suppressions::Suppression> suppressions;
     Settings temp;
 
     guiProject.analyzeAllVsConfigs.clear();
@@ -1068,13 +1082,38 @@ bool ImportProject::importCppcheckGuiProject(std::istream &istr, Settings *setti
                         temp.functionContracts[function] = expects;
                 }
             }
+        } else if (strcmp(node->Name(), CppcheckXml::VariableContractsElementName) == 0) {
+            for (const tinyxml2::XMLElement *child = node->FirstChildElement(); child; child = child->NextSiblingElement()) {
+                if (strcmp(child->Name(), CppcheckXml::VariableContractItemElementName) == 0) {
+                    const char *name = child->Attribute(CppcheckXml::VariableContractVarName);
+                    const char *min = child->Attribute(CppcheckXml::VariableContractMin);
+                    const char *max = child->Attribute(CppcheckXml::VariableContractMax);
+                    if (name)
+                        temp.variableContracts[name] = Settings::VariableContracts{min?min:"", max?max:""};
+                }
+            }
         } else if (strcmp(node->Name(), CppcheckXml::IgnoreElementName) == 0)
             guiProject.excludedPaths = readXmlStringList(node, "", CppcheckXml::IgnorePathName, CppcheckXml::IgnorePathNameAttrib);
         else if (strcmp(node->Name(), CppcheckXml::LibrariesElementName) == 0)
             guiProject.libraries = readXmlStringList(node, "", CppcheckXml::LibraryElementName, nullptr);
-        else if (strcmp(node->Name(), CppcheckXml::SuppressionsElementName) == 0)
-            suppressions = readXmlStringList(node, "", CppcheckXml::SuppressionElementName, nullptr);
-        else if (strcmp(node->Name(), CppcheckXml::VSConfigurationElementName) == 0)
+        else if (strcmp(node->Name(), CppcheckXml::SuppressionsElementName) == 0) {
+            for (const tinyxml2::XMLElement *child = node->FirstChildElement(); child; child = child->NextSiblingElement()) {
+                if (strcmp(child->Name(), CppcheckXml::SuppressionElementName) != 0)
+                    continue;
+                auto read = [](const char *s, const char *def) {
+                    return s ? s : def;
+                };
+                Suppressions::Suppression s;
+                s.errorId = read(child->GetText(), "");
+                s.fileName = read(child->Attribute("fileName"), "");
+                if (!s.fileName.empty())
+                    s.fileName = joinRelativePath(path, s.fileName);
+                s.lineNumber = child->IntAttribute("lineNumber", Suppressions::Suppression::NO_LINE);
+                s.symbolName = read(child->Attribute("symbolName"), "");
+                std::istringstream(read(child->Attribute("hash"), "0")) >> s.hash;
+                suppressions.push_back(s);
+            }
+        } else if (strcmp(node->Name(), CppcheckXml::VSConfigurationElementName) == 0)
             guiProject.checkVsConfigs = readXmlStringList(node, "", CppcheckXml::VSConfigurationName, nullptr);
         else if (strcmp(node->Name(), CppcheckXml::PlatformElementName) == 0)
             guiProject.platform = node->GetText();
@@ -1098,6 +1137,8 @@ bool ImportProject::importCppcheckGuiProject(std::istream &istr, Settings *setti
             temp.checkUnusedTemplates = (strcmp(node->GetText(), "true") == 0);
         else if (strcmp(node->Name(), CppcheckXml::MaxCtuDepthElementName) == 0)
             temp.maxCtuDepth = std::atoi(node->GetText());
+        else if (strcmp(node->Name(), CppcheckXml::MaxTemplateRecursionElementName) == 0)
+            temp.maxTemplateRecursion = std::atoi(node->GetText());
         else if (strcmp(node->Name(), CppcheckXml::CheckUnknownFunctionReturn) == 0)
             ; // TODO
         else if (strcmp(node->Name(), Settings::SafeChecks::XmlRootName) == 0) {
@@ -1113,7 +1154,9 @@ bool ImportProject::importCppcheckGuiProject(std::istream &istr, Settings *setti
                 else
                     return false;
             }
-        } else
+        } else if (strcmp(node->Name(), CppcheckXml::TagWarningsElementName) == 0)
+            ; // TODO
+        else
             return false;
     }
     settings->basePaths = temp.basePaths;
@@ -1128,11 +1171,12 @@ bool ImportProject::importCppcheckGuiProject(std::istream &istr, Settings *setti
 
     for (const std::string &p : paths)
         guiProject.pathNames.push_back(p);
-    for (const std::string &supp : suppressions)
-        settings->nomsg.addSuppressionLine(supp);
+    for (const Suppressions::Suppression &supp : suppressions)
+        settings->nomsg.addSuppression(supp);
     settings->checkHeaders = temp.checkHeaders;
     settings->checkUnusedTemplates = temp.checkUnusedTemplates;
     settings->maxCtuDepth = temp.maxCtuDepth;
+    settings->maxTemplateRecursion = temp.maxTemplateRecursion;
     settings->safeChecks = temp.safeChecks;
     settings->bugHunting = temp.bugHunting;
     settings->functionContracts = temp.functionContracts;
