@@ -131,22 +131,15 @@ public:
         return classDef ? classDef->str() : emptyString;
     }
 
-    bool isClassType() const {
-        return classDef && classDef->str() == "class";
-    }
-
-    bool isEnumType() const {
-        return classDef && classDef->str() == "enum";
-    }
+    bool isClassType() const;
+    bool isEnumType() const;
+    bool isStructType() const;
+    bool isUnionType() const;
 
     bool isTypeAlias() const {
         return classDef && classDef->str() == "using";
     }
 
-    bool isStructType() const {
-        return classDef && classDef->str() == "struct";
-    }
-
     const Token *initBaseInfo(const Token *tok, const Token *tok1);
 
     const Function* getFunction(const std::string& funcName) const;
@@ -199,6 +192,7 @@ class CPPCHECKLIB Variable {
         fIsVolatile   = (1 << 13),  /** @brief volatile */
         fIsSmartPointer = (1 << 14),/** @brief std::shared_ptr|unique_ptr */
         fIsMaybeUnused = (1 << 15), /** @brief marked [[maybe_unused]] */
+        fIsInit       = (1 << 16), /** @brief Is variable initialized in declaration */
     };
 
     /**
@@ -501,6 +495,14 @@ public:
         return getFlag(fHasDefault);
     }
 
+    /**
+     * Is variable initialized in its declaration
+     * @return true if variable declaration contains initialization
+     */
+    bool isInit() const {
+        return getFlag(fIsInit);
+    }
+
     /**
      * Get Type pointer of known type.
      * @return pointer to type if known, NULL if not known
@@ -891,6 +893,7 @@ public:
     const Token *noexceptArg;         ///< noexcept token
     const Token *throwArg;            ///< throw token
     const Token *templateDef;         ///< points to 'template <' before function
+    const Token *functionPointerUsage; ///< function pointer usage
 
     static bool argsMatch(const Scope *scope, const Token *first, const Token *second, const std::string &path, nonneg int path_length);
 
@@ -1040,6 +1043,17 @@ public:
         return false;
     }
 
+    static Function* nestedInFunction(const Scope* scope) {
+        while (scope) {
+            if (scope->type == Scope::eFunction)
+                break;
+            scope = scope->nestedIn;
+        }
+        if (!scope)
+            return nullptr;
+        return scope->function;
+    }
+
     bool isClassOrStruct() const {
         return (type == eClass || type == eStruct);
     }
@@ -1144,6 +1158,11 @@ private:
     void findFunctionInBase(const std::string & name, nonneg int args, std::vector<const Function *> & matches) const;
 };
 
+enum class Reference {
+    None,
+    LValue,
+    RValue
+};
 
 /** Value type */
 class CPPCHECKLIB ValueType {
@@ -1153,6 +1172,7 @@ public:
     nonneg int bits;                    ///< bitfield bitcount
     nonneg int pointer;                 ///< 0=>not pointer, 1=>*, 2=>**, 3=>***, etc
     nonneg int constness;               ///< bit 0=data, bit 1=*, bit 2=**
+    Reference reference = Reference::None;///< Is the outermost indirection of this type a reference or rvalue reference or not? pointer=2, Reference=LValue would be a T**&
     const Scope *typeScope;               ///< if the type definition is seen this point out the type scope
     const ::Type *smartPointerType;       ///< Smart pointer type
     const Token* smartPointerTypeToken;   ///< Smart pointer type token
@@ -1350,6 +1370,7 @@ private:
     void createSymbolDatabaseEnums();
     void createSymbolDatabaseEscapeFunctions();
     void createSymbolDatabaseIncompleteVars();
+    void createSymbolDatabaseExprIds();
 
     void addClassFunction(Scope **scope, const Token **tok, const Token *argStart);
     Function *addGlobalFunctionDecl(Scope*& scope, const Token* tok, const Token *argStart, const Token* funcStart);
