@@ -885,7 +885,7 @@ const Token * Token::findClosingBracket() const
         else if (closing->str() == ">") {
             if (--depth == 0)
                 return closing;
-        } else if (closing->str() == ">>") {
+        } else if (closing->str() == ">>" || closing->str() == ">>=") {
             if (depth <= 2)
                 return closing;
             depth -= 2;
@@ -1053,8 +1053,6 @@ void Token::insertToken(const std::string &tokenStr, const std::string &original
                             scope = tok1->strAt(-3) + " :: " + scope;
                             tok1 = tok1->tokAt(-2);
                         }
-
-                        if (!nextScopeNameAddition.empty() && !scope.empty()) nextScopeNameAddition += " :: ";
                         nextScopeNameAddition += scope;
                     }
                 }
@@ -1142,14 +1140,14 @@ void Token::printOut(const char *title) const
 {
     if (title && title[0])
         std::cout << "\n### " << title << " ###\n";
-    std::cout << stringifyList(true, true, true, true, true, nullptr, nullptr) << std::endl;
+    std::cout << stringifyList(stringifyOptions::forDebugExprId(), nullptr, nullptr) << std::endl;
 }
 
 void Token::printOut(const char *title, const std::vector<std::string> &fileNames) const
 {
     if (title && title[0])
         std::cout << "\n### " << title << " ###\n";
-    std::cout << stringifyList(true, true, true, true, true, &fileNames, nullptr) << std::endl;
+    std::cout << stringifyList(stringifyOptions::forDebugExprId(), &fileNames, nullptr) << std::endl;
 }
 
 void Token::printLines(int lines) const
@@ -1157,12 +1155,12 @@ void Token::printLines(int lines) const
     const Token *end = this;
     while (end && end->linenr() < lines + linenr())
         end = end->next();
-    std::cout << stringifyList(true, true, true, true, true, nullptr, end) << std::endl;
+    std::cout << stringifyList(stringifyOptions::forDebugExprId(), nullptr, end) << std::endl;
 }
 
-void Token::stringify(std::ostream& os, bool varid, bool attributes, bool macro) const
+void Token::stringify(std::ostream& os, const stringifyOptions& options) const
 {
-    if (attributes) {
+    if (options.attributes) {
         if (isUnsigned())
             os << "unsigned ";
         else if (isSigned())
@@ -1174,7 +1172,7 @@ void Token::stringify(std::ostream& os, bool varid, bool attributes, bool macro)
                 os << "long ";
         }
     }
-    if (macro && isExpandedMacro())
+    if (options.macro && isExpandedMacro())
         os << "$";
     if (isName() && mStr.find(' ') != std::string::npos) {
         for (char i : mStr) {
@@ -1191,19 +1189,30 @@ void Token::stringify(std::ostream& os, bool varid, bool attributes, bool macro)
                 os << i;
         }
     }
-    if (varid && mImpl->mVarId != 0)
+    if (options.varid && mImpl->mVarId != 0)
         os << '@' << mImpl->mVarId;
+    if (options.exprid && mImpl->mExprId != 0)
+        os << '@' << mImpl->mExprId;
 }
 
-std::string Token::stringifyList(bool varid, bool attributes, bool linenumbers, bool linebreaks, bool files, const std::vector<std::string>* fileNames, const Token* end) const
+void Token::stringify(std::ostream& os, bool varid, bool attributes, bool macro) const
+{
+    stringifyOptions options;
+    options.varid = varid;
+    options.attributes = attributes;
+    options.macro = macro;
+    stringify(os, options);
+}
+
+std::string Token::stringifyList(const stringifyOptions& options, const std::vector<std::string>* fileNames, const Token* end) const
 {
     if (this == end)
         return "";
 
     std::ostringstream ret;
 
-    unsigned int lineNumber = mImpl->mLineNumber - (linenumbers ? 1U : 0U);
-    unsigned int fileIndex = files ? ~0U : mImpl->mFileIndex;
+    unsigned int lineNumber = mImpl->mLineNumber - (options.linenumbers ? 1U : 0U);
+    unsigned int fileIndex = options.files ? ~0U : mImpl->mFileIndex;
     std::map<int, unsigned int> lineNumbers;
     for (const Token *tok = this; tok != end; tok = tok->next()) {
         bool fileChange = false;
@@ -1213,7 +1222,7 @@ std::string Token::stringifyList(bool varid, bool attributes, bool linenumbers,
             }
 
             fileIndex = tok->mImpl->mFileIndex;
-            if (files) {
+            if (options.files) {
                 ret << "\n\n##file ";
                 if (fileNames && fileNames->size() > tok->mImpl->mFileIndex)
                     ret << fileNames->at(tok->mImpl->mFileIndex);
@@ -1226,18 +1235,18 @@ std::string Token::stringifyList(bool varid, bool attributes, bool linenumbers,
             fileChange = true;
         }
 
-        if (linebreaks && (lineNumber != tok->linenr() || fileChange)) {
+        if (options.linebreaks && (lineNumber != tok->linenr() || fileChange)) {
             if (lineNumber+4 < tok->linenr() && fileIndex == tok->mImpl->mFileIndex) {
                 ret << '\n' << lineNumber+1 << ":\n|\n";
                 ret << tok->linenr()-1 << ":\n";
                 ret << tok->linenr() << ": ";
-            } else if (this == tok && linenumbers) {
+            } else if (this == tok && options.linenumbers) {
                 ret << tok->linenr() << ": ";
             } else {
                 while (lineNumber < tok->linenr()) {
                     ++lineNumber;
                     ret << '\n';
-                    if (linenumbers) {
+                    if (options.linenumbers) {
                         ret << lineNumber << ':';
                         if (lineNumber == tok->linenr())
                             ret << ' ';
@@ -1247,14 +1256,25 @@ std::string Token::stringifyList(bool varid, bool attributes, bool linenumbers,
             lineNumber = tok->linenr();
         }
 
-        tok->stringify(ret, varid, attributes, attributes); // print token
-        if (tok->next() != end && (!linebreaks || (tok->next()->linenr() <= tok->linenr() && tok->next()->fileIndex() == tok->fileIndex())))
+        tok->stringify(ret, options); // print token
+        if (tok->next() != end && (!options.linebreaks || (tok->next()->linenr() <= tok->linenr() && tok->next()->fileIndex() == tok->fileIndex())))
             ret << ' ';
     }
-    if (linebreaks && (files || linenumbers))
+    if (options.linebreaks && (options.files || options.linenumbers))
         ret << '\n';
     return ret.str();
 }
+std::string Token::stringifyList(bool varid, bool attributes, bool linenumbers, bool linebreaks, bool files, const std::vector<std::string>* fileNames, const Token* end) const
+{
+    stringifyOptions options;
+    options.varid = varid;
+    options.attributes = attributes;
+    options.macro = attributes;
+    options.linenumbers = linenumbers;
+    options.linebreaks = linebreaks;
+    options.files = files;
+    return stringifyList(options, fileNames, end);
+}
 
 std::string Token::stringifyList(const Token* end, bool attributes) const
 {
@@ -1437,7 +1457,23 @@ static std::string stringFromTokenRange(const Token* start, const Token* end)
             ret << "unsigned ";
         if (tok->isLong() && !tok->isLiteral())
             ret << "long ";
-        if (tok->originalName().empty() || tok->isUnsigned() || tok->isLong()) {
+        if (tok->tokType() == Token::eString) {
+            for (unsigned char c: tok->str()) {
+                if (c == '\n')
+                    ret << "\\n";
+                else if (c == '\r')
+                    ret << "\\r";
+                else if (c == '\t')
+                    ret << "\\t";
+                else if (c >= ' ' && c <= 126)
+                    ret << c;
+                else {
+                    char str[10];
+                    sprintf(str, "\\x%02x", c);
+                    ret << str;
+                }
+            }
+        } else if (tok->originalName().empty() || tok->isUnsigned() || tok->isLong()) {
             ret << tok->str();
         } else
             ret << tok->originalName();
@@ -1633,6 +1669,12 @@ void Token::printValueFlow(bool xml, std::ostream &out) const
                 case ValueFlow::Value::CONTAINER_SIZE:
                     out << "container-size=\"" << value.intvalue << '\"';
                     break;
+                case ValueFlow::Value::ITERATOR_START:
+                    out << "iterator-start=\"" << value.intvalue << '\"';
+                    break;
+                case ValueFlow::Value::ITERATOR_END:
+                    out << "iterator-end=\"" << value.intvalue << '\"';
+                    break;
                 case ValueFlow::Value::LIFETIME:
                     out << "lifetime=\"" << value.tokvalue << '\"';
                     break;
@@ -1682,6 +1724,12 @@ void Token::printValueFlow(bool xml, std::ostream &out) const
                 case ValueFlow::Value::CONTAINER_SIZE:
                     out << "size=" << value.intvalue;
                     break;
+                case ValueFlow::Value::ITERATOR_START:
+                    out << "start=" << value.intvalue;
+                    break;
+                case ValueFlow::Value::ITERATOR_END:
+                    out << "end=" << value.intvalue;
+                    break;
                 case ValueFlow::Value::LIFETIME:
                     out << "lifetime=" << value.tokvalue->str();
                     break;
@@ -1960,6 +2008,8 @@ bool Token::addValue(const ValueFlow::Value &value)
             case ValueFlow::Value::ValueType::INT:
             case ValueFlow::Value::ValueType::CONTAINER_SIZE:
             case ValueFlow::Value::ValueType::BUFFER_SIZE:
+            case ValueFlow::Value::ValueType::ITERATOR_START:
+            case ValueFlow::Value::ValueType::ITERATOR_END:
                 differentValue = (it->intvalue != value.intvalue);
                 break;
             case ValueFlow::Value::ValueType::TOK:
@@ -2054,6 +2104,8 @@ void Token::type(const ::Type *t)
 
 const ::Type *Token::typeOf(const Token *tok)
 {
+    if (!tok)
+        return nullptr;
     if (Token::simpleMatch(tok, "return")) {
         const Scope *scope = tok->scope();
         if (!scope)
@@ -2074,6 +2126,8 @@ const ::Type *Token::typeOf(const Token *tok)
         if (!function)
             return nullptr;
         return function->retType;
+    } else if (Token::Match(tok->previous(), "%type% (|{")) {
+        return typeOf(tok->previous());
     } else if (Token::simpleMatch(tok, "=")) {
         return Token::typeOf(tok->astOperand1());
     } else if (Token::simpleMatch(tok, ".")) {
@@ -2086,6 +2140,8 @@ const ::Type *Token::typeOf(const Token *tok)
 
 std::pair<const Token*, const Token*> Token::typeDecl(const Token * tok)
 {
+    if (!tok)
+        return {};
     if (Token::simpleMatch(tok, "return")) {
         const Scope *scope = tok->scope();
         if (!scope)
@@ -2102,9 +2158,19 @@ std::pair<const Token*, const Token*> Token::typeDecl(const Token * tok)
             return {};
         if (!var->typeStartToken() || !var->typeEndToken())
             return {};
+        if (Token::simpleMatch(var->typeStartToken(), "auto")) {
+            const Token * tok2 = var->declEndToken();
+            if (Token::Match(tok2, "; %varid% =", var->declarationId()))
+                tok2 = tok2->tokAt(2);
+            if (Token::simpleMatch(tok2, "=") && Token::Match(tok2->astOperand2(), "!!=")) {
+                std::pair<const Token*, const Token*> r = typeDecl(tok2->astOperand2());
+                if (r.first)
+                    return r;
+            }
+        }
         return {var->typeStartToken(), var->typeEndToken()->next()};
-    } else if (Token::Match(tok, "%name%")) {
-        const Function *function = tok->function();
+    } else if (Token::Match(tok->previous(), "%name% (")) {
+        const Function *function = tok->previous()->function();
         if (!function)
             return {};
         return {function->retDef, function->returnDefEnd()};
