@@ -37,6 +37,7 @@ class TimerResults;
 class Token;
 class TemplateSimplifier;
 class ErrorLogger;
+class Preprocessor;
 
 namespace simplecpp {
     class TokenList;
@@ -560,6 +561,15 @@ public:
      */
     static const Token * isFunctionHead(const Token *tok, const std::string &endsWith, bool cpp);
 
+    void setPreprocessor(const Preprocessor *preprocessor) {
+        mPreprocessor = preprocessor;
+    }
+    const Preprocessor *getPreprocessor() const {
+        return mPreprocessor;
+    }
+
+    bool hasIfdef(const Token *start, const Token *end) const;
+
 private:
 
     /**
@@ -607,16 +617,16 @@ private:
 public:
 
     /** Syntax error */
-    void syntaxError(const Token *tok, const std::string &code = "") const;
+    NORETURN void syntaxError(const Token *tok, const std::string &code = "") const;
 
     /** Syntax error. Unmatched character. */
-    void unmatchedToken(const Token *tok) const;
+    NORETURN void unmatchedToken(const Token *tok) const;
 
     /** Syntax error. C++ code in C file. */
-    void syntaxErrorC(const Token *tok, const std::string &what) const;
+    NORETURN void syntaxErrorC(const Token *tok, const std::string &what) const;
 
     /** Warn about unknown macro(s), configuration is recommended */
-    void unknownMacroError(const Token *tok1) const;
+    NORETURN void unknownMacroError(const Token *tok1) const;
 
 private:
 
@@ -733,6 +743,9 @@ private:
      */
     void simplifyOperatorName();
 
+    /** simplify overloaded operators: 'obj(123)' => 'obj . operator() ( 123 )' */
+    void simplifyOverloadedOperators();
+
     /**
     * Remove [[attribute]] (C++11 and later) from TokenList
     */
@@ -956,6 +969,8 @@ private:
     /** Tokenizer maxtime */
     const std::time_t mMaxTime;
 #endif
+
+    const Preprocessor *mPreprocessor;
 };
 
 /// @}
