@@ -47,9 +47,13 @@ private:
         TEST_CASE(expr6);
         TEST_CASE(expr7);
         TEST_CASE(expr8);
+        TEST_CASE(expr9);
         TEST_CASE(exprAssign1);
         TEST_CASE(exprAssign2); // Truncation
 
+        TEST_CASE(inc1);
+        TEST_CASE(inc2);
+
         TEST_CASE(if1);
         TEST_CASE(if2);
         TEST_CASE(if3);
@@ -57,9 +61,13 @@ private:
         TEST_CASE(if5);
         TEST_CASE(ifelse1);
 
+        TEST_CASE(istream);
+
         TEST_CASE(switch1);
         TEST_CASE(switch2);
 
+        TEST_CASE(for1);
+
         TEST_CASE(while1);
         TEST_CASE(while2);
         TEST_CASE(while3);
@@ -70,9 +78,11 @@ private:
         TEST_CASE(array2);
         TEST_CASE(array3);
         TEST_CASE(array4);
+        TEST_CASE(array5);
         TEST_CASE(arrayInit1);
         TEST_CASE(arrayInit2);
         TEST_CASE(arrayUninit);
+        TEST_CASE(arrayInLoop);
 
         TEST_CASE(floatValue1);
         TEST_CASE(floatValue2);
@@ -128,10 +138,10 @@ private:
             replace(line, "(fp.gt ", "(> ");
             replace(line, "(fp.lt ", "(< ");
             int par = 0;
-            for (int pos = 0; pos < line.size(); ++pos) {
-                if (line[pos] == '(')
+            for (char pos : line) {
+                if (pos == '(')
                     par++;
-                else if (line[pos] == ')')
+                else if (pos == ')')
                     --par;
             }
             if (par < 0)
@@ -148,7 +158,7 @@ private:
         std::istringstream istr(code);
         tokenizer.tokenize(istr, "test.cpp");
         std::string ret;
-        std::function<void(const Token *, const ExprEngine::Value &, ExprEngine::DataBase *)> f = [&](const Token *tok, const ExprEngine::Value &value, ExprEngine::DataBase *dataBase) {
+        ExprEngine::Callback f = [&](const Token *tok, const ExprEngine::Value &value, ExprEngine::DataBase *dataBase) {
             if (tok->str() != binop)
                 return;
             const auto *b = dynamic_cast<const ExprEngine::BinOpResult *>(&value);
@@ -191,7 +201,7 @@ private:
         std::istringstream istr(code);
         tokenizer.tokenize(istr, "test.cpp");
         std::string ret;
-        std::function<void(const Token *, const ExprEngine::Value &, ExprEngine::DataBase *)> f = [&](const Token *tok, const ExprEngine::Value &value, ExprEngine::DataBase *dataBase) {
+        ExprEngine::Callback f = [&](const Token *tok, const ExprEngine::Value &value, ExprEngine::DataBase *dataBase) {
             (void)dataBase;
             if ((linenr == 0 || linenr == tok->linenr()) && tok->expressionString() == str) {
                 if (!ret.empty())
@@ -206,18 +216,20 @@ private:
         return ret;
     }
 
-    std::string trackExecution(const char code[]) {
-        Settings settings;
-        settings.bugHunting = true;
-        settings.debugBugHunting = true;
-        settings.platform(cppcheck::Platform::Unix64);
-        settings.library.smartPointers.insert("std::shared_ptr");
-        Tokenizer tokenizer(&settings, this);
+    std::string trackExecution(const char code[], Settings *settings = nullptr) {
+        Settings s;
+        if (!settings)
+            settings = &s;
+        settings->bugHunting = true;
+        settings->debugBugHunting = true;
+        settings->platform(cppcheck::Platform::Unix64);
+        settings->library.smartPointers.insert("std::shared_ptr");
+        Tokenizer tokenizer(settings, this);
         std::istringstream istr(code);
         tokenizer.tokenize(istr, "test.cpp");
         std::vector<ExprEngine::Callback> callbacks;
         std::ostringstream ret;
-        ExprEngine::executeAllFunctions(this, &tokenizer, &settings, callbacks, ret);
+        ExprEngine::executeAllFunctions(this, &tokenizer, settings, callbacks, ret);
         return ret.str();
     }
 
@@ -312,6 +324,22 @@ private:
         expr(code, "==");
     }
 
+    void expr9() {
+        Settings settings;
+        LOAD_LIB_2(settings.library, "std.cfg");
+
+        ASSERT_EQUALS("1:26: $3=0:ffffffff\n"
+                      "1:26: $2=-128:127\n"
+                      "1:27: 0:{ s=($4,[$3],[:]=$2)}\n",
+                      trackExecution("void foo() { std::string s; }", &settings));
+
+
+        ASSERT_EQUALS("1:52: $3=0:ffffffff\n"
+                      "1:52: $2=-128:127\n"
+                      "1:66: 0:{ s=($4,[$3],[:]=$2)}\n",
+                      trackExecution("std::string getName(int); void foo() { std::string s = getName(1); }", &settings));
+    }
+
     void exprAssign1() {
         ASSERT_EQUALS("($1)+(1)", getRange("void f(unsigned char a) { a += 1; }", "a+=1"));
     }
@@ -320,6 +348,24 @@ private:
         ASSERT_EQUALS("2", getRange("void f(unsigned char x) { x = 258; int a = x }", "a=x"));
     }
 
+    void inc1() {
+        ASSERT_EQUALS("(and (>= $1 (- 2147483648)) (<= $1 2147483647))\n"
+                      "(= (+ $1 1) $1)\n"
+                      "z3::unsat\n",
+                      expr("void f(int x) { int y = x++; x == y; }", "=="));
+
+        ASSERT_EQUALS("(and (>= $1 (- 2147483648)) (<= $1 2147483647))\n"
+                      "(= (+ $1 1) (+ $1 1))\n"
+                      "z3::sat\n",
+                      expr("void f(int x) { int y = ++x; x == y; }", "=="));
+    }
+
+    void inc2() {
+        ASSERT_EQUALS("(= 2 2)\n"
+                      "z3::sat\n",
+                      expr("void f() { unsigned char a[2]; a[0] = 1; a[0]++; a[0] == a[0]; }", "=="));
+    }
+
     void if1() {
         ASSERT_EQUALS("(< $1 $2)\n"
                       "(and (>= $1 (- 2147483648)) (<= $1 2147483647))\n"
@@ -384,6 +430,20 @@ private:
     }
 
 
+    void istream() {
+        const char code[] = "void foo(const std::string& in) {\n"
+                            "    std::istringstream istr(in);\n"
+                            "    unsigned short x=5;\n"
+                            "    istr >> x;\n"
+                            "    x==3;\n"
+                            "}";
+
+        ASSERT_EQUALS("(and (>= $1 0) (<= $1 65535))\n"
+                      "(= $1 3)\n"
+                      "z3::sat\n",
+                      expr(code, "=="));
+    }
+
     void switch1() {
         const char code[] = "void f(int x) {\n"
                             "    switch (x) {\n"
@@ -423,6 +483,19 @@ private:
                       expr(code, "=="));
     }
 
+
+    void for1() {
+        const char code[] = "void f() {\n"
+                            "  int x[10];\n"
+                            "  for (int i = 0; i < 10; i++) x[i] = 0;\n"
+                            "  x[4] == 67;\n"
+                            "}";
+        ASSERT_EQUALS("(= 0 67)\n"
+                      "z3::unsat\n",
+                      expr(code, "=="));
+    }
+
+
     void while1() {
         const char code[] = "void f(int y) {\n"
                             "  int x = 0;\n"
@@ -444,7 +517,7 @@ private:
                             "  x == 1;\n"
                             "}";
         ASSERT_EQUALS("(and (>= $2 (- 2147483648)) (<= $2 2147483647))\n"
-                      "(= $2 1)\n"
+                      "(= (+ $2 1) 1)\n"
                       "z3::sat\n",
                       expr(code, "=="));
     }
@@ -508,7 +581,20 @@ private:
                             "void f() { int x = buf[0]; }";
         ASSERT_EQUALS("2:16: $2:0=-2147483648:2147483647\n"
                       "2:20: $2=-2147483648:2147483647\n"
-                      "2:26: { buf=($1,size=10,[:]=$2) x=$2:0}\n",
+                      "2:26: 0:{ buf=($1,[10],[:]=$2) x=$2:0}\n",
+                      trackExecution(code));
+    }
+
+    void array5() {
+        const char code[] = "int f(int x) {\n"
+                            "  int buf[3][4][5];\n"
+                            "  buf[x][1][2] = 10;\n"
+                            "  return buf[0][1][2];\n"
+                            "}";
+        ASSERT_EQUALS("1:14: $1=-2147483648:2147483647\n"
+                      "1:14: 0:{ x=$1}\n"
+                      "2:19: 0:{ x=$1 buf=($2,[3][4][5],[:]=?)}\n"
+                      "3:20: 0:{ x=$1 buf=($2,[3][4][5],[:]=?,[((20)*($1))+(7)]=10)}\n",
                       trackExecution(code));
     }
 
@@ -524,6 +610,15 @@ private:
         ASSERT_EQUALS("?", getRange("int f() { int arr[10]; return arr[4]; }", "arr[4]"));
     }
 
+    void arrayInLoop() {
+        const char code[] = "void f() {\n"
+                            "  int arr[3][3];\n"
+                            "  for (int i = 0; i < 3; i++) arr[i][0] = arr[1][2];\n"
+                            "  return arr[0][0];"
+                            "}";
+        ASSERT_EQUALS("?", getRange(code, "arr[1][2]"));
+    }
+
 
     void floatValue1() {
         ASSERT_EQUALS("-inf:inf", getRange("float f; void func() { f=f; }", "f=f"));
@@ -591,7 +686,7 @@ private:
 
     void pointer1() {
         const char code[] = "void f(unsigned char *p) { return *p == 7; }";
-        ASSERT_EQUALS("size=$1,[:]=?,null", getRange(code, "p"));
+        ASSERT_EQUALS("[$1],[:]=?,null", getRange(code, "p"));
         ASSERT_EQUALS("(and (>= $3 0) (<= $3 255))\n"
                       "(= $3 7)\n"
                       "z3::sat\n",
