@@ -47,6 +47,9 @@ private:
         TEST_CASE(importCompileCommands2); // #8563
         TEST_CASE(importCompileCommands3); // check with existing trailing / in directory
         TEST_CASE(importCompileCommands4); // only accept certain file types
+        TEST_CASE(importCompileCommands5); // Windows/CMake/Ninja generated comile_commands.json
+        TEST_CASE(importCompileCommands6); // Windows/CMake/Ninja generated comile_commands.json with spaces
+        TEST_CASE(importCompileCommands7);
         TEST_CASE(importCompileCommandsArgumentsSection); // Handle arguments section
         TEST_CASE(importCompileCommandsNoCommandSection); // gracefully handles malformed json
         TEST_CASE(importCppcheckGuiProject);
@@ -99,9 +102,11 @@ private:
     }
 
     void importCompileCommands1() const {
-        const char json[] = "[ { \"directory\": \"/tmp\","
-                            "\"command\": \"gcc -I/tmp -DFILESDIR=\\\\\\\"/usr/local/share/Cppcheck\\\\\\\" -DTEST1 -DTEST2=2 -o /tmp/src.o -c /tmp/src.c\","
-                            "\"file\": \"/tmp/src.c\" } ]";
+        const char json[] = R"([{
+                                   "directory": "/tmp",
+                                   "command": "gcc -DFILESDIR=\"/usr/local/share/Cppcheck\" -DTEST1 -DTEST2=2 -o /tmp/src.o -c /tmp/src.c",
+                                   "file": "/tmp/src.c"
+                               }])";
         std::istringstream istr(json);
         TestImporter importer;
         importer.importCompileCommands(istr);
@@ -110,9 +115,11 @@ private:
     }
 
     void importCompileCommands2() const {
-        const char json[] = "[ { \"directory\": \"/tmp\","
-                            "\"command\": \"gcc -c src.c\","
-                            "\"file\": \"src.c\" } ]";
+        const char json[] = R"([{
+                                   "directory": "/tmp",
+                                   "command": "gcc -c src.c",
+                                   "file": "src.c"
+                               }])";
         std::istringstream istr(json);
         TestImporter importer;
         importer.importCompileCommands(istr);
@@ -121,9 +128,11 @@ private:
     }
 
     void importCompileCommands3() const {
-        const char json[] = "[ { \"directory\": \"/tmp/\","
-                            "\"command\": \"gcc -c src.c\","
-                            "\"file\": \"src.c\" } ]";
+        const char json[] = R"([{
+                                    "directory": "/tmp/",
+                                    "command": "gcc -c src.c",
+                                    "file": "src.c"
+                               }])";
         std::istringstream istr(json);
         TestImporter importer;
         importer.importCompileCommands(istr);
@@ -132,15 +141,71 @@ private:
     }
 
     void importCompileCommands4() const {
-        const char json[] = "[ { \"directory\": \"/tmp/\","
-                            "\"command\": \"gcc -c src.mm\","
-                            "\"file\": \"src.mm\" } ]";
+        const char json[] = R"([{
+                                    "directory": "/tmp/",
+                                    "command": "gcc -c src.mm",
+                                    "file": "src.mm"
+                               }])";
         std::istringstream istr(json);
         TestImporter importer;
         importer.importCompileCommands(istr);
         ASSERT_EQUALS(0, importer.fileSettings.size());
     }
 
+    void importCompileCommands5() const {
+        const char json[] =
+            R"([{
+                "directory": "C:/Users/dan/git/build-test-cppcheck-Desktop_Qt_5_15_0_MSVC2019_64bit-Debug",
+                "command": "C:\\PROGRA~2\\MICROS~1\\2019\\COMMUN~1\\VC\\Tools\\MSVC\\1427~1.291\\bin\\HostX64\\x64\\cl.exe /nologo /TP -IC:\\Users\\dan\\git\\test-cppcheck\\mylib\\src /DWIN32 /D_WINDOWS /GR /EHsc /Zi /Ob0 /Od /RTC1 -MDd -std:c++17 /Fomylib\\CMakeFiles\\mylib.dir\\src\\foobar\\mylib.cpp.obj /FdTARGET_COMPILE_PDB /FS -c C:\\Users\\dan\\git\\test-cppcheck\\mylib\\src\\foobar\\mylib.cpp",
+                "file": "C:\\Users\\dan\\git\\test-cppcheck\\mylib\\src\\foobar\\mylib.cpp"
+             },
+             {
+                "directory": "C:/Users/dan/git/build-test-cppcheck-Desktop_Qt_5_15_0_MSVC2019_64bit-Debug",
+                "command": "C:\\PROGRA~2\\MICROS~1\\2019\\COMMUN~1\\VC\\Tools\\MSVC\\1427~1.291\\bin\\HostX64\\x64\\cl.exe /nologo /TP -IC:\\Users\\dan\\git\\test-cppcheck\\myapp\\src -Imyapp -IC:\\Users\\dan\\git\\test-cppcheck\\mylib\\src /DWIN32 /D_WINDOWS /GR /EHsc /Zi /Ob0 /Od /RTC1 -MDd -std:c++17 /Fomyapp\\CMakeFiles\\myapp.dir\\src\\main.cpp.obj /FdTARGET_COMPILE_PDB /FS -c C:\\Users\\dan\\git\\test-cppcheck\\myapp\\src\\main.cpp",
+                "file": "C:\\Users\\dan\\git\\test-cppcheck\\myapp\\src\\main.cpp"
+             }])";
+        std::istringstream istr(json);
+        TestImporter importer;
+        importer.importCompileCommands(istr);
+        ASSERT_EQUALS(2, importer.fileSettings.size());
+        ASSERT_EQUALS("C:/Users/dan/git/test-cppcheck/mylib/src/", importer.fileSettings.begin()->includePaths.front());
+    }
+
+    void importCompileCommands6() const {
+        const char json[] =
+            R"([{
+                "directory": "C:/Users/dan/git/build-test-cppcheck-Desktop_Qt_5_15_0_MSVC2019_64bit-Debug",
+                "command": "C:\\PROGRA~2\\MICROS~1\\2019\\COMMUN~1\\VC\\Tools\\MSVC\\1427~1.291\\bin\\HostX64\\x64\\cl.exe /nologo /TP -IC:\\Users\\dan\\git\\test-cppcheck\\mylib\\src -I\"C:\\Users\\dan\\git\\test-cppcheck\\mylib\\second src\" /DWIN32 /D_WINDOWS /GR /EHsc /Zi /Ob0 /Od /RTC1 -MDd -std:c++17 /Fomylib\\CMakeFiles\\mylib.dir\\src\\foobar\\mylib.cpp.obj /FdTARGET_COMPILE_PDB /FS -c C:\\Users\\dan\\git\\test-cppcheck\\mylib\\src\\foobar\\mylib.cpp",
+                "file": "C:\\Users\\dan\\git\\test-cppcheck\\mylib\\src\\foobar\\mylib.cpp"
+            },
+            {
+                "directory": "C:/Users/dan/git/build-test-cppcheck-Desktop_Qt_5_15_0_MSVC2019_64bit-Debug",
+                "command": "C:\\PROGRA~2\\MICROS~1\\2019\\COMMUN~1\\VC\\Tools\\MSVC\\1427~1.291\\bin\\HostX64\\x64\\cl.exe /nologo /TP -IC:\\Users\\dan\\git\\test-cppcheck\\myapp\\src -Imyapp -IC:\\Users\\dan\\git\\test-cppcheck\\mylib\\src /DWIN32 /D_WINDOWS /GR /EHsc /Zi /Ob0 /Od /RTC1 -MDd -std:c++17 /Fomyapp\\CMakeFiles\\myapp.dir\\src\\main.cpp.obj /FdTARGET_COMPILE_PDB /FS -c C:\\Users\\dan\\git\\test-cppcheck\\myapp\\src\\main.cpp",
+                "file": "C:\\Users\\dan\\git\\test-cppcheck\\myapp\\src\\main.cpp"
+             }])";
+        std::istringstream istr(json);
+        TestImporter importer;
+        importer.importCompileCommands(istr);
+        ASSERT_EQUALS(2, importer.fileSettings.size());
+        ASSERT_EQUALS("C:/Users/dan/git/test-cppcheck/mylib/second src/", importer.fileSettings.begin()->includePaths.front());
+    }
+
+
+    void importCompileCommands7() const {
+        const char json[] = R"([{
+                                   "directory": "/tmp",
+                                   "command": "gcc -DFILESDIR=\"\\\"/home/danielm/cppcheck 2\\\"\" -I\"/home/danielm/cppcheck 2/build/externals/tinyxml\" -DTEST1 -DTEST2=2 -o /tmp/src.o -c /tmp/src.c",
+                                   "file": "/tmp/src.c"
+                               }])";
+        std::istringstream istr(json);
+        TestImporter importer;
+        importer.importCompileCommands(istr);
+        ASSERT_EQUALS(1, importer.fileSettings.size());
+        //FIXME ASSERT_EQUALS("FILESDIR=\"/home/danielm/cppcheck 2\";TEST1=1;TEST2=2", importer.fileSettings.begin()->defines);
+        ASSERT_EQUALS(1, importer.fileSettings.begin()->includePaths.size());
+        ASSERT_EQUALS("/home/danielm/cppcheck 2/build/externals/tinyxml/", importer.fileSettings.begin()->includePaths.front());
+    }
+
     void importCompileCommandsArgumentsSection() const {
         const char json[] = "[ { \"directory\": \"/tmp/\","
                             "\"arguments\": [\"gcc\", \"-c\", \"src.c\"],"
